# MIT license
# Copyright 2023 Sergej Alikov <sergej@alikov.com>

import argparse
import importlib.metadata
import logging
import time

import adafruit_scd30  # type: ignore
import board  # type: ignore
import busio  # type: ignore
from prometheus_client import Gauge, Summary, start_http_server

logger = logging.getLogger(__name__)

METRIC_MEASUREMENT_TIME = Summary(
    "scd30_measurement_seconds", "Time spent processing performing measurement"
)
METRIC_SENSOR_CO2 = Gauge("scd30_sensor_co2", "CO2 (PPM)")
METRIC_SENSOR_TEMPERATURE = Gauge("scd30_sensor_temperature", "Temperature (degrees C)")
METRIC_SENSOR_HUMIDITY = Gauge("scd30_sensor_humidity", "Humidity (%%rH)")


def parse_args() -> argparse.Namespace:
    parser = argparse.ArgumentParser(
        prog="scd30-exporter",
        description=importlib.metadata.metadata("scd30-exporter")["Summary"],
    )

    parser.add_argument(
        "--log-level",
        default="WARNING",
        choices=("CRITICAL", "ERROR", "WARNING", "INFO", "DEBUG", "NOTSET"),
        help="log level",
    )

    parser.add_argument(
        "--interval", type=int, default=10, help="interval in seconds between readings"
    )

    parser.add_argument("--port", type=int, default=8000, help="exporter port")

    return parser.parse_args()


@METRIC_MEASUREMENT_TIME.time()
def read_data(scd: adafruit_scd30.SCD30) -> tuple:
    while True:
        if scd.data_available:
            return scd.CO2, scd.temperature, scd.relative_humidity

        time.sleep(0.5)


def main() -> None:

    args = parse_args()

    logging.basicConfig(
        format="%(asctime)s %(levelname)s %(message)s",
        level=args.log_level,
        datefmt="%Y-%m-%d %H:%M:%S",
    )

    start_http_server(args.port)

    # SCD-30 has tempremental I2C with clock stretching, datasheet recommends
    # starting at 50KHz
    i2c = busio.I2C(board.SCL, board.SDA, frequency=50000)
    scd = adafruit_scd30.SCD30(i2c)

    while True:
        co2, temp, humidity = read_data(scd)
        logger.debug(
            f"CO2: {co2:.2f} PPM, Temperature: {temp:.2f} °C, Humidity: {humidity:.2f} %%rH"
        )
        METRIC_SENSOR_CO2.set(co2)
        METRIC_SENSOR_TEMPERATURE.set(temp)
        METRIC_SENSOR_HUMIDITY.set(humidity)

        time.sleep(args.interval)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['scd30_exporter']

package_data = \
{'': ['*']}

install_requires = \
['adafruit-circuitpython-scd30>=2.2.9,<3.0.0',
 'prometheus-client>=0.16.0,<0.17.0']

entry_points = \
{'console_scripts': ['scd30-exporter = scd30_exporter.cli:main']}

setup_kwargs = {
    'name': 'scd30-exporter',
    'version': '0.1.0',
    'description': 'Prometheus exporter for Adafruit SCD-30 - NDIR CO2 Temperature and Humidity Sensor',
    'long_description': '# scd30-exporter\n\nPrometheus exporter for Adafruit SCD-30 - NDIR CO2 Temperature and Humidity Sensor.\n\n\n## Installing\n\n```sh\npip3 install scd30-exporter\n```\n\n## Running\n\n```sh\nscd30-exporter --port 8000 --interval 10\n```\n\n\n## Developing\n\nSee [DEVELOPING.md](DEVELOPING.md)\n',
    'author': 'Sergej Alikov',
    'author_email': 'sergej@alikov.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/node13h/scd30-exporter',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
